#!/bin/bash
#

# need an in source build

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e9:p2710|e9:p2711|e10:p2710|e10:p2711> <debug|opt|prof> [tar]"
   echo "        p2711 -> python v2_7_11"
   echo "        p2710 -> python v2_7_10"
   echo "        e10 -> gcc v4_9_3a"
   echo "        e9 -> gcc v4_9_3"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}

set_qualifiers()
{
 
    if [ -z ${basequal} ]
    then
       echo "ERROR: please specify the qualifiers"
       usage
       exit 1
    fi
    if [ -z ${extraqual} ]
    then
       echo "ERROR: please specify the qualifiers"
       usage
       exit 1
    fi

    fullqual=${basequal}:${extraqual}
    qualdir=`echo ${basequal} | sed -e 's/:/-/g'`-${extraqual}
}

get_platform_name()
{
    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
       flvr=`ups flavor -2`
    fi
    OS=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform`
    plat=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform osplat`

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${product_dir}/${package}/${pkgver}/${flvr}-${qualdir}
    patchdir=${product_dir}/${package}/${pkgver}/patch
    blddir=${product_dir}/${package}/${pkgver}/build/${flvr}-${qualdir}
    tardir=${product_dir}/${package}/${pkgver}/tar
    sourcedir=${product_dir}/${package}/${pkgver}/source
}

die()
{
  echo "$1"; exit 2
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=nucondb
origpkgver=v1_4_20
pkgver=${origpkgver}
ssibuildshims_version=v0_09_02
pkgtarfile=${package}-${origpkgver}.tar 

get_this_dir

get_ssibuildshims

set_qualifiers

get_platform_name

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

case ${basequal} in
e9:p2710) stdcxx="-std=c++1y" ;;
e9:p2711) stdcxx="-std=c++1y" ;;
e10:p2710) stdcxx="-std=c++14" ;;
e10:p2711) stdcxx="-std=c++14" ;;
*) 
   echo "ERROR: unrecognized qualifier ${basequal}"
   usage
   exit 1
esac

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi

cd ${pkgdir} 
tar xf ${tardir}/${pkgtarfile} || die "ERROR: failed to unwind ${pkgtarfile}"
rm -rf build || die "ERROR: cannot remove old build subdirectory"

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || die "ERROR: fake setup failed"

if [ -z ${LIBWDA_DIR} ]
then
   echo "ERROR: failed to setup libwda"
   exit 1
else
   echo "LIBWDA_DIR: ${LIBWDA_DIR}"
fi
if [ -z ${IFDHC_DIR} ]
then
   echo "ERROR: failed to setup ifdhc"
   exit 1
else
   echo "IFDHC_DIR: ${IFDHC_DIR}"
fi

echo

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

case ${extraqual} in
debug) ARCH="${stdcxx} -O0 -g";;
prof)  ARCH="${stdcxx} -g -p -pg";;
opt)   ARCH="${stdcxx} -O2";;
esac
export ARCH

set -x

cd ${pkgdir}/src

make all || die "ERROR: make all failed"
make install || die "ERROR: make install failed"
make clean || die "ERROR: make clean failed"

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi


# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS} || die "ERROR: setup failed"

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
